@@ -29,12 +29,38 @@ use XML::DOM;
 
 use EmailReminder::Utils;
 
-require Exporter;
-our @ISA = ("Exporter");
-our @EXPORT = qw(is_occurring get_recipients);
+# XML tags
+__PACKAGE__->mk_accessors(qw(name));
 
-# XML tags, attributes and values
+sub mk_accessors {
+    my ($self, @field_names) = @_;
 
+    # get the classname since it might be a derived class calling this
+    my $class = ref $self || $self;
+
+    foreach my $field_name ( @field_names ) {
+        my $get_method = sub {
+            my ($self) = @_;
+            my $valid_sub = "valid_$field_name";
+            my $value = EmailReminder::Utils::get_node_value($self->{XML_NODE}, $field_name);
+            return $self->$valid_sub( $value );
+        };
+
+        my $set_method = sub {
+            my ($self, $new_value) = @_;
+            return EmailReminder::Utils::set_node_value($self->{XML_NODE}, $field_name, $new_value);
+        };
+
+        {
+            no strict 'refs';
+            *{"${class}::get_$field_name"} = $get_method;
+            *{"${class}::set_$field_name"} = $set_method;
+        }
+    }
+    return 1;
+}
+
+# other XML tags, attributes and values
 my $REMINDER_TAG = 'reminder';
 my $REMINDERS_TAG = 'reminders';
 my $RECIPIENT_TAG = 'recipient';
@@ -50,7 +76,7 @@ my $SAME_DAY_VAL = 'same day';
 # Hard-coded value for this event's type (class method)
 sub get_type
 {
-    return undef;
+    return;
 }
 
 # Number of fields this event adds to its parent (class method)
@@ -59,91 +85,104 @@ sub get_nb_fields
     return 1;
 }
 
+sub valid_name {
+    my ($class, $new_value) = @_;
+    return $new_value;
+}
+
 sub new
 {
     my $class = shift;
     my $event_node = shift;
     my $id = shift;
 
-    my $this = { "OCCURRING" => 0,
+    my $self = { "OCCURRING" => 0,
                  "XML_NODE" => $event_node,
                  "ID" => $id,
                  "DATA" => [$id],
             };
     
-    bless $this, $class;
+    bless $self, $class;
 
     # Create empty data array
-    my $count = $this->get_nb_fields() - 1;
+    my $count = $self->get_nb_fields() - 1;
     for (my $i = 0; $i < $count; $i++) {
-        push(@{$this->{DATA}}, undef);
+        push(@{$self->{DATA}}, undef);
     }
 
     # Where to send this reminder email
-    my $recipients = $this->{XML_NODE}->getElementsByTagName($RECIPIENTS_TAG)->item(0);
+    my $recipients = $self->{XML_NODE}->getElementsByTagName($RECIPIENTS_TAG)->item(0);
     if (defined($recipients)) {
-        $this->{RECIPIENTS_NODE} = $recipients;
-        $this->{RECIPIENTS_CACHE} = $this->get_recipients();
+        $self->{RECIPIENTS_NODE} = $recipients;
+        $self->{RECIPIENTS_CACHE} = $self->get_recipients();
     }
 
     # Process reminders
-    my $reminders = $this->{XML_NODE}->getElementsByTagName($REMINDERS_TAG)->item(0);
+    my $reminders = $self->{XML_NODE}->getElementsByTagName($REMINDERS_TAG)->item(0);
     if (defined($reminders)) {
-        $this->{REMINDERS_NODE} = $reminders;
-        $this->{REMINDERS_CACHE} = $this->get_reminders();
+        $self->{REMINDERS_NODE} = $reminders;
+        $self->{REMINDERS_CACHE} = $self->get_reminders();
     }
 
-    return $this;
+    return $self;
 }
 
-sub unlink
+sub unlink_event
 {
-    my $this = shift;
-    my $node = $this->{XML_NODE};
+    my $self = shift;
+    my $node = $self->{XML_NODE};
     $node->getParentNode()->removeChild($node);
     $node->dispose();
+    return 1;
 }
 
 sub get_recipients
 {
-    my $this = shift;
+    my $self = shift;
 
-    if (!defined($this->{RECIPIENTS_CACHE})) {
+    if (!defined($self->{RECIPIENTS_CACHE})) {
         my @recipients = ();
 
-        if (defined($this->{RECIPIENTS_NODE})) {
-            foreach my $recipient ($this->{RECIPIENTS_NODE}->getElementsByTagName($RECIPIENT_TAG)) {
+        if (defined($self->{RECIPIENTS_NODE})) {
+            foreach my $recipient ($self->{RECIPIENTS_NODE}->getElementsByTagName($RECIPIENT_TAG)) {
                 my $email = $recipient->getAttribute($EMAIL_ATTR);
                 if (defined($email)) {
-                    my $name = $recipient->getAttribute($NAME_ATTR);
-                    push(@recipients, [$email, $name]);
+                    my $fname = undef;
+                    my $lname = undef;
+                    my $fullname = $recipient->getAttribute($NAME_ATTR);
+
+                    my @name_parts = split(/ /, $fullname);
+                    $fname = $name_parts[0];
+                    $lname = $name_parts[-1] if @name_parts > 1;
+
+                    push(@recipients, [$email, $fname, $lname]);
                 }
             }
         }
         
-        $this->{RECIPIENTS_CACHE} = \@recipients;
+        $self->{RECIPIENTS_CACHE} = \@recipients;
     }
-    return $this->{RECIPIENTS_CACHE};
+    return $self->{RECIPIENTS_CACHE};
 }
 
 sub get_reminders
 {
-    my $this = shift;
+    my $self = shift;
 
-    if (!defined($this->{REMINDERS_CACHE})) {
+    if (!defined($self->{REMINDERS_CACHE})) {
         my @reminders = ();
 
-        if (defined($this->{REMINDERS_NODE}))
+        if (defined($self->{REMINDERS_NODE}))
         {
-            foreach my $reminder ($this->{REMINDERS_NODE}->getElementsByTagName($REMINDER_TAG)){
+            foreach my $reminder ($self->{REMINDERS_NODE}->getElementsByTagName($REMINDER_TAG)){
                 my $type = $reminder->getAttribute($TYPE_ATTR);
                 
                 if ($type eq $SAME_DAY_VAL) {
                     push(@reminders, 0);
                     
-                    if ($this->will_occur("")) {
-                        $this->{WHEN} = "today";
-                        $this->{OCCURRING}++;
+                    if ($self->will_occur("")) {
+                        $self->{WHEN} = "today";
+                        $self->{OCCURRING}++;
                     }
                 }
                 elsif (($type eq $DAYS_BEFORE_VAL) && 
@@ -152,47 +191,47 @@ sub get_reminders
                     my $days = $reminder->getFirstChild()->getNodeValue();
                     push(@reminders, $days);
                     
-                    if ($this->will_occur($days)) {
+                    if ($self->will_occur($days)) {
                         if ($days > 1) {
                             my $upcoming_date = DateCalc("today", "+${days}days");
-                            $this->{WHEN} = "in $days days (" . UnixDate($upcoming_date, "%A %b %e") . ")";
+                            $self->{WHEN} = "in $days days (" . UnixDate($upcoming_date, "%A %b %e") . ")";
+                        }
+                        elsif ($days == 1) {
+                            $self->{WHEN} = "tomorrow";
                         }
-                        elsif ($days == 1) {		
-                            $this->{WHEN} = "tomorrow";
-                        }		
                         elsif ($days == 0) {
-                            $this->{WHEN} = "today";
+                            $self->{WHEN} = "today";
                         }
                         else { 
                             next; # Negative days are ignored
                         } 
                         
-                        $this->{OCCURRING}++;
+                        $self->{OCCURRING}++;
                     }
                 }
             }
         }
 
-        $this->{REMINDERS_CACHE} = \@reminders;
+        $self->{REMINDERS_CACHE} = \@reminders;
     }
 
-    return $this->{REMINDERS_CACHE};
+    return $self->{REMINDERS_CACHE};
 }
 
 sub set_reminders
 {
-    my ($this, $new_reminders) = @_;
+    my ($self, $new_reminders) = @_;
 
-    my $event = $this->{XML_NODE};
+    my $event = $self->{XML_NODE};
     my $doc = $event->getOwnerDocument();
-    my $reminders = $this->{REMINDERS_NODE};
+    my $reminders = $self->{REMINDERS_NODE};
 
     if (!defined($reminders)) {
         # Create a blank <reminders> tag
         $reminders = $doc->createElement($REMINDERS_TAG);
         $event->appendChild($reminders);
 
-        $this->{REMINDERS_NODE} = $reminders;
+        $self->{REMINDERS_NODE} = $reminders;
     }
     else {
         # TODO: preserve extra reminders in the XML but not in the UI
@@ -227,28 +266,36 @@ sub set_reminders
     }
 
     # Clear the cache
-    undef $this->{REMINDERS_CACHE};
+    undef $self->{REMINDERS_CACHE};
+
+    return 1;
 }
 
 sub is_occurring
 {
-    my $this = shift;
-    return $this->{OCCURRING};
+    my $self = shift;
+    return $self->{OCCURRING};
 }
 
 sub get_message
 {
-    my $this = shift;
-    my $body = $this->get_message_body(@_);
+    my $self = shift;
+
+    # destination user
+    my $first_name = shift || 'there';
+
+    my $body = $self->get_message_body(@_);
 
     my $message = "";
-    $message = <<MESSAGEEND if $body;
+    $message = <<"MESSAGEEND" if $body;
+Hi $first_name,
+
 $body
 Have a good day!
 
 --
 Sent by Email-Reminder $EmailReminder::Utils::VERSION
-http://people.debian.org/~francois/email-reminder/
+http://www.email-reminder.org.nz/
 MESSAGEEND
 
     return $message;
@@ -256,14 +303,14 @@ MESSAGEEND
 
 sub data
 {
-    my $this = shift;
-    return $this->{DATA};
+    my $self = shift;
+    return $self->{DATA};
 }
 
 sub get_id
 {
-    my $this = shift;
-    return $this->{ID};
+    my $self = shift;
+    return $self->{ID};
 }
 
 1;
