@@ -19,7 +19,7 @@ package EmailReminder::WeeklyEvent;
 
 use strict;
 use warnings;
-
+use overload '""' => \&str;
 use Date::Manip;
 use POSIX;
 use Scalar::Util;
@@ -27,19 +27,21 @@ use Scalar::Util;
 use EmailReminder::Event;
 use EmailReminder::Utils;
 
-require Exporter;
-our @ISA    = ("EmailReminder::Event", "Exporter");
-#our @EXPORT = qw(get_name valid_day);
+use base qw(EmailReminder::Event);
 
-# XML tags, attributes and values
-my $DAY_TAG   = 'day';
-my $NAME_TAG  = 'name';
+# XML tags
+__PACKAGE__->mk_accessors(qw(day));
 
 # Global date variables
 my $current_time = ParseDate("now");
 my $current_date = ParseDate(UnixDate($current_time, "\%x"));
 my $current_dayofweek = UnixDate($current_time, "\%w");
 
+sub str {
+    my ($self) = @_;
+    return $self->get_type . ':' . $self->get_id . ') ' . $self->get_name . ' - ' . $self->get_day;
+}
+
 # Hard-coded value for this event's type (class method)
 sub get_type
 {
@@ -49,25 +51,13 @@ sub get_type
 # Number of fields this event adds to its parent (class method)
 sub get_nb_fields
 {
-    return EmailReminder::Event->get_nb_fields() + 2;
-}
-
-sub get_name
-{
-    my ($this) = @_;
-    my $name  = EmailReminder::Utils::get_node_value($this->{XML_NODE}, $NAME_TAG);
-    return $name;
-}
-
-sub set_name
-{
-    my ($this, $new_value) = @_;
-    return EmailReminder::Utils::set_node_value($this->{XML_NODE}, $NAME_TAG, $new_value);
+    my ($self) = @_;
+    return $self->SUPER::get_nb_fields() + 2;
 }
 
 sub valid_day
 {
-    my ($new_value) = @_;
+    my ($class, $new_value) = @_;
 
     if (!Scalar::Util::looks_like_number($new_value)) {
         # Try to parse as a string
@@ -88,39 +78,21 @@ sub valid_day
     return $new_value;
 }
 
-sub get_day
-{
-    my ($this) = @_;
-    my $day = EmailReminder::Utils::get_node_value($this->{XML_NODE}, $DAY_TAG);
-    return valid_day($day);
-}
-
-sub set_day
-{
-    my ($this, $new_value) = @_;
-    return EmailReminder::Utils::set_node_value($this->{XML_NODE}, $DAY_TAG, valid_day($new_value));
-}
-
 sub get_subject
 {
-    my $this = shift;
-    return $this->get_name();
+    my $self = shift;
+    return $self->get_name();
 }
 
 sub get_message_body
 {
-    my $this = shift;
-
-    # destination user
-    my $first_name = shift;
+    my $self = shift;
 
     # event details
-    my $when      = $this->{"WHEN"};
-    my $name      = $this->get_name();
+    my $when      = $self->{"WHEN"};
+    my $name      = $self->get_name();
     
-    my $message = <<MESSAGEEND;
-Hi $first_name,
-
+    my $message = <<"MESSAGEEND";
 I just want to remind you of the following event $when:
 
 $name
@@ -132,11 +104,11 @@ MESSAGEEND
 # Returns 1 if the event will occur in X days (X is a param)
 sub will_occur
 {
-    my $this = shift;
+    my $self = shift;
     my $modifier = shift;
     
     # Apply the modifier to the event date
-    my $modified_day = $this->get_day();
+    my $modified_day = $self->get_day();
     return 0 unless $modified_day;
 
     if ($modifier) {
